@@ -1,3 +1,4 @@
+import enum
 import json
 import re
 import weakref
@@ -13,13 +14,14 @@ from urllib.parse import urlencode
 from reprlib import repr
 
 from requests.utils import CaseInsensitiveDict, to_key_val_list
-from pydantic import parse_obj_as, BaseModel as PydanticBaseModel
+from pydantic import BaseModel as PydanticBaseModel
 from cachetools import TTLCache
 from loguru import logger
 
 from deepfos.cache import Manager, AppSeperatedLRUCache
+from deepfos.api.models import compat_parse_obj_as as parse_obj_as
 from deepfos.lib.httpcli import AioHttpCli
-from deepfos.lib.utils import concat_url, retry, to_version_tuple, repr_version
+from deepfos.lib.utils import concat_url, retry, to_version_tuple, repr_version, trim_text
 from deepfos.lib.asynchronous import evloop
 from deepfos.lib.discovery import ServiceDiscovery
 from deepfos.lib.constant import UNSET, RE_SYS_SERVER_PARSER
@@ -38,6 +40,12 @@ T_DictPydanticModel = Union[dict, PydanticBaseModel]
 VERSIONED_MODULE = "deepfos.api.V{version}.{name}"
 
 
+class ContentType(str, enum.Enum):
+    json = "json"
+    bytes = "bytes"
+    text = "text"
+
+
 class RequestInfo:
     """请求参数wrapper，提供参数的展示，合并，构造功能"""
     _repr_attr = ['url', 'method', 'header', 'body']
@@ -304,6 +312,7 @@ class Route:
         allow_none: bool = True,
         raise_false: bool = True,
         data_wrapped: bool = True,
+        content_type: Union[ContentType, str] = ContentType.json
     ):
         """
         装饰器主入口，同时是所有API接口的实际入口。
@@ -347,6 +356,11 @@ class Route:
                 else:
                     model = default_model
 
+                if 'content_type' in kwargs:
+                    _content_type = ContentType(kwargs.pop('content_type'))
+                else:
+                    _content_type = content_type
+
                 _retries = kwargs.pop('retries', retries)
                 use_cache = kwargs.pop('use_cache', False)
 
@@ -393,16 +407,16 @@ class Route:
                     else:
                         # ------------------------------------------------------------
                         # parse content type
-                        if (content_type := header.get('content-type')) is None:
+                        if (ctype := header.get('content-type')) is None:
                             logger.warning('Missing content-type in request header.')
-                        elif matched := RE_CONTENT_TYPE.match(content_type):
-                            content_type = matched.group('ctype').lower()
-                        elif self._RE_MULTI_TYPE.match(content_type):
-                            content_type = 'data'
+                        elif matched := RE_CONTENT_TYPE.match(ctype):
+                            ctype = matched.group('ctype').lower()
+                        elif self._RE_MULTI_TYPE.match(ctype):
+                            ctype = 'data'
                         else:
-                            logger.warning(f'Unknow content-type: {content_type}')
+                            logger.warning(f'Unknow content-type: {ctype}')
 
-                        body_key = content_type if content_type in ('json', 'data') else 'body'
+                        body_key = ctype if ctype in ('json', 'data') else 'body'
 
                     req_args = {
                         'url': url,
@@ -425,7 +439,13 @@ class Route:
 
                 # -----------------------------------------------------------------------------
                 # parse response
+                if _content_type is ContentType.bytes:
+                    return await resp.read()
+
                 text = await resp.text()
+                if _content_type is ContentType.text:
+                    return text
+
                 err_code = None
 
                 if not 200 <= (status_code := resp.status) < 300:
@@ -434,7 +454,7 @@ class Route:
                         "Detail: {text}.",
                         url=lambda: req.url,
                         code=lambda: status_code,
-                        text=lambda: text,
+                        text=lambda: trim_text(text, OPTION.general.response_display_length_on_error),
                     )
                     flag, obj, err = False, None, f"[code: {status_code}] ErrMsg from server: {text}"
 
@@ -455,7 +475,7 @@ class Route:
                                     "Call API: {url} failed. "
                                     "Bad response because status is False. Detail: {text}.",
                                     url=lambda: req.url,
-                                    text=lambda: text,
+                                    text=lambda: trim_text(text, OPTION.general.response_display_length_on_error),
                                 )
                                 flag, obj, err = False, resp.get('data'), resp.get('message', text)
                                 err_code = resp.get('code')
@@ -706,7 +726,7 @@ class RootAPI:
 
     if TYPE_CHECKING:  # pragma: no cover
         # 由APIBase带入，此处定义仅用于ide提示
-        header: str = ''
+        header: Dict[str, str] = {}
         base_url: str = ''
 
     @classmethod
