@@ -410,12 +410,20 @@ class SQLCondition:
             val_list = self.val_list.copy()
         elif 1 == len(fd_me) == len(fd_other):
             val_list = pd.DataFrame()
-            self.__static_cond = self.__static_cond.append(
-                pd.DataFrame(data={'field': self.fields[0], 'op': [OpFactory(IN, self.val_list.iloc[:, 0])]})
-            )
-            other.__static_cond = other.__static_cond.append(
-                pd.DataFrame(data={'field': other.fields[0], 'op': [OpFactory(IN, other.val_list.iloc[:, 0])]})
-            )
+            self.__static_cond = pd.concat([
+                self.__static_cond,
+                pd.DataFrame(data={
+                    'field': self.fields[0],
+                    'op': [OpFactory(IN, self.val_list.iloc[:, 0])]
+                })
+            ])
+            other.__static_cond = pd.concat([
+                other.__static_cond,
+                pd.DataFrame(data={
+                    'field': other.fields[0],
+                    'op': [OpFactory(IN, other.val_list.iloc[:, 0])]
+                })
+            ])
         else:
             # 字段越多，限制条件越多，一般数据量越小，定义为sub
             if fd_me.issubset(fd_other):
@@ -465,8 +473,14 @@ class SQLCondition:
 
         if len(fields) == 1:
             # 将条件归类至sta_cond
-            static_cond = self.__static_cond.append(
-                pd.DataFrame(data={'field': fields, 'op': [OpFactory(IN, val_list.iloc[:, 0])]})
+            static_cond = pd.concat(
+                [
+                    self.__static_cond,
+                    pd.DataFrame(data={
+                        'field': fields,
+                        'op': [OpFactory(IN, val_list.iloc[:, 0])]
+                    })
+                ]
             ).sort_values('field').reset_index(drop=True)
             val_list = pd.DataFrame()
         else:
