@@ -339,11 +339,12 @@ class MetaTable(MetaNodeMixin):
 
         df = pd.DataFrame()
         for cd in cond.to_sql():
-            df = df.append(cls.datatable.select(
-                columns=fields,
-                where=cd,
-                distinct=unique,
-            ))
+            df = pd.concat(
+                [
+                    df,
+                    cls.datatable.select(columns=fields, where=cd, distinct=unique)
+                ]
+            )
 
         logger.debug(f"Got DATA:\n {df}")
 
