@@ -57,8 +57,8 @@ def dataframe_to_records(
 
 
 class TableNode(MetaTable):
-    table_structure: TableStructure = None
-    name: str = None
+    table_structure: Optional[TableStructure] = None
+    name: Optional[str] = None
 
 
 def create_table(struct: TableStructure, parent: BaseTable = None) -> TableNode:
@@ -171,10 +171,10 @@ class AsyncBusinessModel(ElementBase[BusinessModelAPI]):
         path: str = None,
         server_name: str = None,
     ):
-        super().__init__(element_name, folder_id, path, server_name)
         self.__tables: Dict[str, TableNode] = {}
         self.__logic_tables = LogicTable()
         self.__dflt_ptn_id = UNSET
+        super().__init__(element_name, folder_id, path, server_name)
 
     @cached_property
     def default_partition_id(self) -> Optional[str]:
@@ -646,7 +646,7 @@ class AsyncBusinessModel(ElementBase[BusinessModelAPI]):
             df_row = pd.DataFrame([{'process_operation_id': operation_info.id,
                                     'origin_status': operation_info.originStatusList,
                                     'target_status': operation_info.targetStatus}])
-            df_operation = df_operation.append(df_row)
+            df_operation = pd.concat([df_operation, df_row])
         df_operation.reset_index(drop=True, inplace=True)
         if len(df_operation) > 1:
             df_operation['target_status_shift'] = df_operation['target_status'].shift(1)
@@ -699,7 +699,7 @@ class AsyncBusinessModel(ElementBase[BusinessModelAPI]):
                 df_success['line_no'] += 1
                 df_success['result_status'] = row['target_status']
                 df_success['process_operation_id'] = row['process_operation_id']
-                df_insert = df_insert.append(df_success)
+                df_insert = pd.concat([df_insert, df_success])
 
             tbl_main = self.logic_tables[main_tbl_name].async_datatable
             if not df_failure.empty:
