@@ -593,8 +593,8 @@ class AsyncDataTableMySQL(ElementBase):
         table_name: str = None,
         server_name: str = None,
     ):
-        super().__init__(element_name, folder_id, path, server_name)
         self.__tbl_name = table_name
+        super().__init__(element_name, folder_id, path, server_name)
 
     def _safe_get_txn_conf(self) -> _TxnConfig:
         try:
@@ -1187,12 +1187,11 @@ class AsyncDataTableMySQL(ElementBase):
         if key_where not in source.columns:
             raise ValueError(f"Column <{key_where}> is missing in source dataframe.")
 
-        where_col = source[key_where]
         valid_columns = source.columns.intersection(self.structure.columns.keys())
-
         table = self.table
 
         def yield_sql(df):
+            where_col = df[key_where]
             for idx, upd_data in enumerate(df[valid_columns].to_dict(orient='records')):
                 q = self.query.update(table)
 
@@ -1691,7 +1690,7 @@ class AsyncDirectAccessDataTableMySQL(AsyncDataTableMySQL):
             limit=limit,
             offset=offset
         )
-        return raw_data.to_dict(orient='record')
+        return raw_data.to_dict(orient='records')
 
     @txn_support
     async def run_sql(self, sql: str):
@@ -2155,7 +2154,7 @@ class AsyncDataTablePostgreSQL(AsyncDataTableMySQL):
     __doc__ = DOC_TEMPLATE.format(DB='PostgreSQL')
     api: PostgreSQLAPI
     api_class = PostgreSQLAPI
-    quote_char = ''
+    quote_char = '"'
     convertor = _PostgreSQLDFConvertor(quote_char=quote_char)
     query = PostgreSQLQuery
 
@@ -2237,8 +2236,6 @@ class AsyncDataTableKingBase(AsyncDataTablePostgreSQL):
     __doc__ = DOC_TEMPLATE.format(DB='KingBase')
     api: KingBaseAPI
     api_class = KingBaseAPI
-    quote_char = '"'
-    convertor = _PostgreSQLDFConvertor(quote_char=quote_char)
 
 
 class DataTableKingBase(
@@ -2328,8 +2325,6 @@ class AsyncDataTableDeepModelKingBase(AsyncDataTableKingBase):
     __doc__ = DOC_TEMPLATE.format(DB='DeepModelKingBase')
     api: DeepModelKingBaseAPI
     api_class = DeepModelKingBaseAPI
-    quote_char = ''
-    convertor = _PostgreSQLDFConvertor(quote_char=quote_char)
 
 
 class DataTableDeepModelKingBase(
