@@ -8,9 +8,9 @@ import pandas as pd
 from loguru import logger
 
 from deepfos.element.rolestrategy import RoleStrategy, AsyncRoleStrategy
-from pydantic import parse_obj_as, Field
 from multidict import CIMultiDict
 
+from deepfos.api.models import compat_parse_obj_as as parse_obj_as
 from .base import ElementBase, SyncMeta
 from .datatable import AsyncDataTableMySQL, get_table_class, T_AsyncDatatableClass
 from deepfos.algo.graph import DAGraph
@@ -38,11 +38,13 @@ from deepfos.lib.utils import unpack_expr, CIEnum, get_language_key_map
 from deepfos.lib.asynchronous import future_property
 from deepfos.lib.constant import (
     ROOT, ACCEPT_LANS, DFLT_NAME_COLUMN,
-    DFLT_PNAME_COLUMN, SHAREDMEMBER, INDEX_FIELD
+    DFLT_PNAME_COLUMN, SHAREDMEMBER, INDEX_FIELD,
+    SHAREDMEMBERV12, DFLT_PNAME_COLUMN_V12
 )
 
 __all__ = [
     'DimMember',
+    'DimMemberV12',
     'Dimension',
     'AsyncDimension'
 ]
@@ -55,24 +57,84 @@ class Strategy(CIEnum):
 
 
 class DimMember(DimensionMemberOperationSw):
-    is_active = True
-    is_modula = False
-    aggweight = 1.0
-    sharedmember = False
-    datatype = "NUMBER"
-    parent_name = ROOT
+    is_active: bool = True
+    is_modula: bool = False
+    aggweight: int = 1.0
+    sharedmember: bool = False
+    datatype: str = "NUMBER"
+    parent_name: str = ROOT
 
 
-class DimMemberV12(DimensionMemberOperationSw):
-    """仅用alias特性便于做字段兼容"""
-    is_active = Field(True, alias='isActive')
-    is_modula = False
-    aggweight = 1.0
-    sharedmember = Field(False, alias='sharedMember')
-    datatype = "NUMBER"
-    parent_name = Field(ROOT, alias='parentName')
-    origin_name: str = Field(None, alias='originName')
-    origin_parent_name: str = Field(None, alias='originParentName')
+class DimMemberV12(DimensionMemberOperationDto):
+    isActive: bool = True
+    aggweight: int = 1.0
+    sharedMember: bool = False
+    dataType: str = "NUMBER"
+    parentName: str = ROOT
+
+    @property
+    def parent_name(self):
+        return self.parentName
+
+    @property
+    def sharedmember(self):
+        return self.sharedMember
+
+    @property
+    def origin_name(self):
+        return self.originName
+
+    @property
+    def origin_parent_name(self):
+        return self.originParentName
+
+    @property
+    def is_active(self):
+        return self.isActive
+
+    @property
+    def accounttype(self):
+        return self.accountType
+
+    @property
+    def datatype(self):
+        return self.dataType
+
+    @property
+    def end_period(self):
+        return self.endPeriod
+
+    @property
+    def end_year(self):
+        return self.endYear
+
+    @property
+    def flowtype(self):
+        return self.flowType
+
+    @property
+    def halfyear(self):
+        return self.halfYear
+
+    @property
+    def local_currency(self):
+        return self.localCurrency
+
+    @property
+    def period_level(self):
+        return self.periodLevel
+
+    @property
+    def start_year(self):
+        return self.startYear
+
+    @property
+    def start_period(self):
+        return self.startPeriod
+
+    @property
+    def sort_col(self):
+        return self.sortCol
 
 
 ADD = 'add'
@@ -81,6 +143,29 @@ DEL = 'delete'
 
 OPE = 'operation'
 
+V10_TO_V12 = {
+    'accounttype': 'accountType',
+    'datatype': 'dataType',
+    'end_period': 'endPeriod',
+    'end_year': 'endYear',
+    'flowtype': 'flowType',
+    'halfyear': 'halfYear',
+    'is_active': 'isActive',
+    'local_currency': 'localCurrency',
+    'parent_name': 'parentName',
+    'period_level': 'periodLevel',
+    'origin_name': 'originName',
+    'origin_parent_name': 'originParentName',
+    'sharedmember': 'sharedMember',
+    'start_period': 'startPeriod',
+    'start_year': 'startYear',
+    'sort_col': 'sortCol',
+    'is_modula': 'is_modula',
+    'is_base': 'is_base',
+    'is_calculated': 'is_calculated',
+    'formula': 'formula',
+}
+
 
 # -----------------------------------------------------------------------------
 # core
@@ -100,12 +185,12 @@ class AsyncDimension(ElementBase[DimensionAPI]):
         strict: bool = False,
         server_name: str = None,
     ):
-        super().__init__(element_name, folder_id, path, server_name)
         self._add_memo = CIMultiDict()
         self._del_memo = CIMultiDict()
         self._upd_memo = CIMultiDict()
         self.__member_memo = None if strict else CIMultiDict()
         self._strict = strict
+        super().__init__(element_name, folder_id, path, server_name)
 
     @cached_property
     def _datatable_class(self) -> T_AsyncDatatableClass:
@@ -155,19 +240,9 @@ class AsyncDimension(ElementBase[DimensionAPI]):
 
         return self.__member_memo
 
-    @staticmethod
-    def _get_parent(
-        member: DimMember,
-        memo: CIMultiDict,
-    ) -> Optional[DimensionMemberOperationSw]:
-        """获取维度成员的父节点，父节点为root不可使用此方法"""
-        parent_name = member.parent_name
-        parent_candidate = memo.getall(parent_name, [])
-
-        if not parent_candidate:
-            return None
-
-        return parent_candidate[0]
+    @cached_property
+    def is_v1_0_api(self) -> bool:
+        return self.api.version == (1, 0)
 
     @future_property(on_demand=True)
     async def members(self) -> List[DimensionMembersDto]:
@@ -184,7 +259,6 @@ class AsyncDimension(ElementBase[DimensionAPI]):
             name=self.element_name
         )
         if api.version == api.default_version:
-
             mbr = await api.custom.select_dimension_member_list(dim_info)
         else:
             mbr = await api.extra.dimension_custom_select_dimension_member_list(dim_info)
@@ -201,7 +275,11 @@ class AsyncDimension(ElementBase[DimensionAPI]):
             return await api.query.open_dimension_info_by_id(ele_info)
         return await api.extra.dimension_query_open_dimension_info_by_id(ele_info)
 
-    def add_member(self, *members: DimMember, silent: bool = False) -> 'AsyncDimension':
+    def add_member(
+        self, 
+        *members: Union[DimMember, DimMemberV12], 
+        silent: bool = False
+    ) -> 'AsyncDimension':
         """增加维度成员，
 
         新增的维度成员暂时维护在当前的维度对象中，
@@ -305,7 +383,11 @@ class AsyncDimension(ElementBase[DimensionAPI]):
         self._member_memo.extend(new_value)
         return self
 
-    def reorder_members(self, memo: CIMultiDict[DimMember], complete: bool) -> CIMultiDict[DimMember]:
+    def reorder_members(
+        self, 
+        memo: CIMultiDict[Union[DimMember, DimMemberV12]], 
+        complete: bool
+    ) -> CIMultiDict[Union[DimMember, DimMemberV12]]:
         """成员重排序
 
         对维度成员按照先父节点后子节点的顺序排序
@@ -376,24 +458,21 @@ class AsyncDimension(ElementBase[DimensionAPI]):
 
         await self._finalize_update()
 
-        is_v1_0 = self.api.version == self.api.default_version
         dim_members = []
         for v in chain(
             self._add_memo.values(),
             self._upd_memo.values(),
             self._del_memo.values()
         ):
-            mbr = v.dict(exclude_none=True)
-            if is_v1_0:
-                mbr.setdefault(INDEX_FIELD, 0)
-                dim_members.append(mbr)
+            mbr = v
+            if self.is_v1_0_api:
+                mbr.index = mbr.index or 0
             else:
-                dim_members.append(DimensionMemberOperationDto.construct_from(
-                    **DimMemberV12.construct_from(**mbr).dict(by_alias=True)
-                ))
+                mbr = self._maybe_rename_fields(v)
+            dim_members.append(mbr.dict(exclude_none=True))
 
         if len(dim_members) > 0:
-            if is_v1_0:
+            if self.is_v1_0_api:
                 payload = DimensionMemberChangeSaveSw.construct_from(
                     dimensionInfo=DimensionInfoSw(
                         name=self.element_info.elementName,
@@ -404,12 +483,50 @@ class AsyncDimension(ElementBase[DimensionAPI]):
                 )
                 r = await self._incr_save_members(payload)
             else:
-                r = await self._save_tree(dim_members)
+                r = await self._save_tree(
+                    parse_obj_as(List[DimensionMemberOperationDto], dim_members)
+                )
             self._add_memo.clear()
             self._del_memo.clear()
             self._upd_memo.clear()
             return r
 
+    def _maybe_rename_fields(
+        self,
+        mbr: Union[
+            DimensionMemberOperationSw,
+            DimensionMemberOperationDto,
+            Dict[str, str]
+        ]
+    ) -> Union[
+        DimensionMemberOperationSw,
+        DimensionMemberOperationDto,
+        Dict[str, str]
+    ]:
+        """
+        Maybe rename fields in V1.0 dimension mbr model
+        to v1.1 & v1.2 model when current api is v1.1 or v1.2
+        """
+        if self.is_v1_0_api:
+            return mbr
+
+        if isinstance(mbr, DimensionMemberOperationDto):
+            return mbr
+
+        return_model = False
+        if isinstance(mbr, DimensionMemberOperationSw):
+            return_model = True
+            mbr = mbr.dict(exclude_none=True)
+
+        for old, new in V10_TO_V12.items():
+            if old in mbr:
+                mbr[new] = mbr.pop(old)
+
+        if return_model:
+            return DimMemberV12.construct_from(**mbr)
+
+        return mbr
+
     async def _incr_save_members(self, payload: DimensionMemberChangeSaveSw):
         resp = await self.async_api.save.incremental(payload)
         if resp.errors:
@@ -425,7 +542,9 @@ class AsyncDimension(ElementBase[DimensionAPI]):
         role_name: str = None,
         role_group: str = None,
         multi_entity_config: List[Dict] = None
-    ) -> Union[List[DimensionMemberBean], List[Dict[str, Any]], List[MultiEntityConfigResult]]:
+    ) -> Union[
+        List[DimensionMemberBean], List[Dict[str, Any]], List[MultiEntityConfigResult]
+    ]:
         """查询成员
 
         根据维度表达式查询维度成员
@@ -504,10 +623,15 @@ class AsyncDimension(ElementBase[DimensionAPI]):
               内部的字典将不包含未查询的属性
 
         """
-        payload = self._get_query_payload(expression, fields, role, role_name, role_group, multi_entity_config)
-        if self.api.version == self.api.default_version:
+        payload = self._get_query_payload(
+            expression, fields, role, role_name, role_group, multi_entity_config
+        )
+        if self.is_v1_0_api:
             api_call = self.async_api.query.select_dimension_member_by_name_function
-        elif self.api.version == (1, 2) and as_model and multi_entity_config is not None:
+        elif (
+            self.api.version == (1, 2)
+            and as_model and multi_entity_config is not None
+        ):
             api_call = functools.partial(
                 self.async_api.extra.dimension_query_select_dimension_member_by_name_function,
                 resp_model=List[MultiEntityConfigResult]
@@ -541,12 +665,17 @@ class AsyncDimension(ElementBase[DimensionAPI]):
 
             rs_mapping = 1
             for dim in role.meta.dimensions:
-                if dim.elementName == self.element_name and dim.folderId == self.element_info.folderId:
+                if (
+                    dim.elementName == self.element_name
+                    and dim.folderId == self.element_info.folderId
+                ):
                     break
                 rs_mapping += 1
             if rs_mapping > len(role.meta.dimensions):
-                raise ValueError(f"Current Dimension: {self.element_name} doesn't exist in "
-                                 f"role strategy: {role.element_name}.")
+                raise ValueError(
+                    f"Current Dimension: {self.element_name} doesn't exist in "
+                    f"role strategy: {role.element_name}."
+                )
 
             role_params.update({'roleFolderId': role.element_info.folderId,
                                 'rsMapping': rs_mapping,
@@ -587,14 +716,26 @@ class AsyncDimension(ElementBase[DimensionAPI]):
 
         Args:
             member_name: 维度成员名
-            **attrs: 成员属性，可接受参数参考 :class:`DimMember`
+
+            **attrs: 成员属性，可接受参数参考:
+
+                - 维度1.0 : :class:`DimMember`
+                - 维度1.1或1.2 : :class:`DimMemberV12`
 
         Returns:
             self
         """
+        attrs = self._maybe_rename_fields(attrs)
         if member_name in self._add_memo:
             upd_mbr = self._add_memo[member_name]
-            self._add_memo[member_name] = DimMember.construct_from(upd_mbr, **attrs)
+            if isinstance(upd_mbr, DimensionMemberOperationSw):
+                self._add_memo[member_name] = DimMember.construct_from(
+                    upd_mbr, **attrs
+                )
+            else:
+                self._add_memo[member_name] = DimMemberV12.construct_from(
+                    upd_mbr, **attrs
+                )
         elif member_name in self._del_memo:
             raise MemberNotFoundError(f"Member: {member_name} does not exist.")
         elif self._strict and member_name not in self._member_memo:
@@ -657,10 +798,7 @@ class AsyncDimension(ElementBase[DimensionAPI]):
 
         strategy = Strategy[strategy]
         df = dataframe.copy().reset_index(drop=True)
-        if (
-            INDEX_FIELD not in df.columns
-            and self.api.version == self.api.default_version
-        ):
+        if INDEX_FIELD not in df.columns and self.is_v1_0_api:
             df[INDEX_FIELD] = 0
 
         # validate dataframe
@@ -676,8 +814,16 @@ class AsyncDimension(ElementBase[DimensionAPI]):
         lan_columns = list(set(df.columns).intersection(ACCEPT_LANS))
         if lan_columns:
             df['multilingual'] = df[lan_columns].to_dict(orient='records')
+
+        mbrcls = DimMember if self.is_v1_0_api else DimMemberV12
+
+        if not self.is_v1_0_api:
+            df = df.rename(columns=V10_TO_V12)
+
         # pick up valid columns
-        valid_columns = list(set(DimMember.__fields__).intersection(df.columns))
+        valid_columns = list(
+            set(mbrcls.__fields__).intersection(df.columns)
+        )
 
         # save replace
         if strategy is Strategy.full_replace:
@@ -686,6 +832,7 @@ class AsyncDimension(ElementBase[DimensionAPI]):
 
         # split dataframe by existed data
         upd_df, add_df = await self._split_dimension_df(df[valid_columns])
+
         # modify operation column
         with self._local_memo():
             if strategy is Strategy.incr_replace:
@@ -695,35 +842,30 @@ class AsyncDimension(ElementBase[DimensionAPI]):
             for record in add_df.to_dict(orient='records'):
                 self._add_memo.add(
                     record[DFLT_NAME_COLUMN],
-                    DimMember.construct_from(**record, operation=ADD)
+                    mbrcls.construct_from(**record, operation=ADD)
                 )
 
             return await self.save(reorder=reorder)
 
-    async def _save_replace(self, reorder, member_list):
+    async def _save_replace(
+        self, reorder: bool, member_list: List[Dict[str, str]]
+    ):
+        mbrcls = DimMember if self.is_v1_0_api else DimMemberV12
+
         if reorder:
             mbrs = CIMultiDict()
             for rec in member_list:
-                mbrs.add(rec[DFLT_NAME_COLUMN], DimMember.construct_from(**rec))
+                mbrs.add(rec[DFLT_NAME_COLUMN], mbrcls.construct_from(**rec))
             mbrs = self.reorder_members(mbrs, complete=True)
             member_list = [mbr.dict(exclude_none=True) for mbr in mbrs.values()]
 
-        is_v1_0 = self.api.version == self.api.default_version
-        if not is_v1_0:
-            member_list = [
-                DimMemberV12.construct_from(**mbr).dict(
-                    exclude_none=True, by_alias=True
-                )
-                for mbr in member_list
-            ]
-
         payload = DimMemberSave.construct_from(
             self.element_info,
             dimensionName=self.element_name,
             increment=0,
             dimensionMemberList=member_list
         )
-        if is_v1_0:
+        if self.is_v1_0_api:
             resp = await self.async_api.member.save(payload)
         elif self.api.version == (1, 1):
             resp = await self.async_api.save.save(payload)
@@ -744,11 +886,14 @@ class AsyncDimension(ElementBase[DimensionAPI]):
         new_memo = CIMultiDict()
 
         for mbr in mbrs:
+            # Use DimMember since query response model fits
             upd_mbr = DimMember.construct_from(mbr)
             upd_mbr.operation = UPD
             upd_mbr.origin_name = upd_mbr.name
             upd_mbr.origin_parent_name = upd_mbr.parent_name
 
+            upd_mbr = self._maybe_rename_fields(upd_mbr)
+
             for attrs in upd_memo.getall(upd_mbr.name, []):
                 if DFLT_NAME_COLUMN in attrs:
                     # 移除被改名的原始成员
@@ -789,7 +934,13 @@ class AsyncDimension(ElementBase[DimensionAPI]):
         self,
         df: pd.DataFrame
     ) -> Tuple[pd.DataFrame, pd.DataFrame]:
-        if SHAREDMEMBER not in df.columns:
+        shared_mbr_col = SHAREDMEMBER
+        pname_col = DFLT_PNAME_COLUMN
+        if not self.is_v1_0_api:
+            shared_mbr_col = SHAREDMEMBERV12
+            pname_col = DFLT_PNAME_COLUMN_V12
+
+        if shared_mbr_col not in df.columns:
             if self._strict:
                 existed_mbrs = self._member_memo
             else:
@@ -805,14 +956,14 @@ class AsyncDimension(ElementBase[DimensionAPI]):
 
         # -----------------------------------------------------------------------------
         # has shared member
-        df_shared: pd.DataFrame = df[df[SHAREDMEMBER]]
+        df_shared: pd.DataFrame = df[df[shared_mbr_col].astype(bool)]
         if self._strict:
             existed_mbrs = self._member_memo
 
             dup_sharedmbrs = []
             for idx, row in df_shared.iterrows():
-                for mbr in self._member_memo.getall(row[DFLT_NAME_COLUMN]):
-                    if mbr.sharedmember and mbr.parent_name == row[DFLT_PNAME_COLUMN]:
+                for mbr in self._member_memo.getall(row[DFLT_NAME_COLUMN], []):
+                    if mbr.sharedmember and mbr.parent_name == row[pname_col]:
                         dup_sharedmbrs.append(idx)
                         break
         else:
@@ -832,13 +983,13 @@ class AsyncDimension(ElementBase[DimensionAPI]):
             shared_memo = set(tuple(item) for item in existed_shared.to_numpy())
             dup_sharedmbrs = []
             for idx, row in df_shared.iterrows():
-                if (row[DFLT_NAME_COLUMN], row[DFLT_PNAME_COLUMN]) in shared_memo:
+                if (row[DFLT_NAME_COLUMN], row[pname_col]) in shared_memo:
                     dup_sharedmbrs.append(idx)
 
         # 在df中丢弃系统中已存在的共享节点
         df_shared_remain = df_shared[~df_shared.index.isin(dup_sharedmbrs)]
 
-        df_not_shared = df[~df[SHAREDMEMBER]]
+        df_not_shared = df[~df[shared_mbr_col].astype(bool)]
         existed_idx = df_not_shared[DFLT_NAME_COLUMN].isin(existed_mbrs)
         return df_not_shared.loc[existed_idx], \
             pd.concat([df_not_shared.loc[~existed_idx], df_shared_remain])
@@ -868,15 +1019,20 @@ class AsyncDimension(ElementBase[DimensionAPI]):
             data = data.rename(columns=field_map)
 
         # 父子关系与aggweight为层级关系所需信息
-        parent = data[['name', 'parent_name', 'aggweight']]
+        parent = data[[DFLT_NAME_COLUMN, DFLT_PNAME_COLUMN, 'aggweight']]
         # parent_name为#root，即只作parent的记录
-        parent = parent[~(parent['parent_name'] == '#root')]
+        parent = parent[~(parent[DFLT_PNAME_COLUMN] == '#root')]
         # 共享成员的父子关系重复，去重
         # 保证source -> target关系唯一性
-        parent = parent.drop_duplicates(subset=['name', 'parent_name'], keep='first')
-        parent = parent.rename(columns={'name': 'source', 'parent_name': 'target'})
+        parent = parent.drop_duplicates(
+            subset=[DFLT_NAME_COLUMN, DFLT_PNAME_COLUMN],
+            keep='first'
+        )
+        parent = parent.rename(
+            columns={DFLT_NAME_COLUMN: 'source', DFLT_PNAME_COLUMN: 'target'}
+        )
         # 主数据排除共享成员
-        data = data.drop_duplicates(subset=['name'], keep='first')
+        data = data.drop_duplicates(subset=[DFLT_NAME_COLUMN], keep='first')
         from deepfos.element.deepmodel import AsyncDeepModel
         dm = AsyncDeepModel()
         async with dm.start_transaction():
@@ -962,7 +1118,7 @@ class AsyncDimension(ElementBase[DimensionAPI]):
         if 'shared_member' not in dataframe.columns:
             dataframe = dataframe.assign(shared_member=False)
 
-        records = dataframe[fields].to_dict(orient="records")
+        records = dataframe[list(fields)].to_dict(orient="records")
         mbrs = []
         for record in records:
             mbrs.append(
@@ -1016,7 +1172,9 @@ def _validate_df_for_dimension(df: pd.DataFrame):
         ).size()
         duplicated = unique_df[unique_df['size'] > 1]
         if not duplicated.empty:
-            raise ValueError(f"Duplicated member name for: {duplicated.to_dict(orient='record')}")
+            raise ValueError(
+                f"Duplicated member name for: {duplicated.to_dict(orient='records')}"
+            )
     else:
         if df[DFLT_NAME_COLUMN].nunique() != len(df):
             counter = Counter(df[DFLT_NAME_COLUMN])
@@ -1060,7 +1218,11 @@ class Dimension(AsyncDimension, metaclass=SyncMeta):
             role_name: str = None,
             role_group: str = None,
             multi_entity_config: List[Dict] = None
-        ) -> Union[List[DimensionMemberBean], List[Dict[str, Any]], List[MultiEntityConfigResult]]:  # pragma: no cover
+        ) -> Union[
+            List[DimensionMemberBean],
+            List[Dict[str, Any]],
+            List[MultiEntityConfigResult]
+        ]:  # pragma: no cover
             ...
 
         def to_deepmodel_object(
