@@ -1,7 +1,7 @@
 import uuid
 
-from pydantic import parse_obj_as
-
+from deepfos.api.models import compat_parse_obj_as as parse_obj_as
+from deepfos.element.datatable import get_table_class
 from deepfos.exceptions import (
     JournalModelSaveError, JournalModelCheckError, JournalModelPostingError
 )
@@ -57,8 +57,6 @@ _escape_table = {
 
 class AsyncJournalModel(ElementBase[JournalModelAPI]):
     """凭证组件"""
-    quote_char = '`'
-
     def __init__(
         self,
         element_name: str,
@@ -66,8 +64,8 @@ class AsyncJournalModel(ElementBase[JournalModelAPI]):
         path: str = None,
         server_name: str = None,
     ):
-        super().__init__(element_name, folder_id, path, server_name)
         self.__tbl_name = None
+        super().__init__(element_name, folder_id, path, server_name)
 
     @future_property
     async def config(self) -> JournalModelConfig:
@@ -77,6 +75,15 @@ class AsyncJournalModel(ElementBase[JournalModelAPI]):
         res = await api.journal_model_data.get_config(element_info)
         return res
 
+    @cached_property
+    def quote_char(self) -> str:
+        try:
+            element_type = self.config.logicTable.dataTableInfo.elementDetail.elementType
+            dbcls = get_table_class(element_type)
+            return dbcls.quote_char
+        except Exception: # noqa
+            return '`'
+
     @cached_property
     def table_name(self) -> str:
         """数据表真实表名"""
@@ -95,7 +102,12 @@ class AsyncJournalModel(ElementBase[JournalModelAPI]):
             `pypika的github <https://github.com/kayak/pypika#tables-columns-schemas-and-databases>`_
 
         """
-        return Table(self.table_name)
+        try:
+            element_type = self.config.logicTable.dataTableInfo.elementDetail.elementType
+            dbcls = get_table_class(element_type)
+            return Table(self.table_name, query_cls=dbcls.query)
+        except Exception: # noqa
+            return Table(self.table_name)
 
     def _parse_where(self, where: Union[None, Term, EmptyCriterion]) -> str:
         if isinstance(where, (Term, EmptyCriterion)):
@@ -110,8 +122,7 @@ class AsyncJournalModel(ElementBase[JournalModelAPI]):
         line_df: pd.DataFrame,
         callback: Union[Dict, CallbackInfo] = None,
         relation_field: str = 'journal_id',
-        id_col: str = None,
-        operate_type: Literal['EDIT', 'ADD'] = 'ADD',
+        id_col: str = MAIN_ID,
         enable_create: bool = True,
         enable_default_value: bool = True,
         enable_repeat_check: bool = True,
@@ -119,9 +130,10 @@ class AsyncJournalModel(ElementBase[JournalModelAPI]):
         enable_valid_range: bool = True,
         enable_all_errors: bool = True,
         enable_need_one_line: bool = True,
+        header_operate: Literal['EDIT', 'ADD', 'DELETE_ADD'] = 'ADD',
+        line_operate: Literal['EDIT', 'ADD', 'DELETE', 'DELETE_ADD'] = 'ADD',
     ):
         errors = set()
-        is_editing = operate_type == 'EDIT'
 
         if head_df.empty:
             errors.add('凭证头表数据DataFrame不能为空')
@@ -131,6 +143,7 @@ class AsyncJournalModel(ElementBase[JournalModelAPI]):
             errors.add('凭证头行表的关联字段relation_field不能为空')
 
         self._maybe_raise_errors(errors)
+        is_editing = not (header_operate == 'ADD' and line_operate == 'ADD')
 
         if is_editing:
             head_required_fields = [relation_field, '_type', id_col]
@@ -181,10 +194,20 @@ class AsyncJournalModel(ElementBase[JournalModelAPI]):
             )
         else:
             head_data = head_data.rename(
-                columns={"_type": "journalTypeCode", relation_field: "headMainId"}
+                columns={"_type": "journalTypeCode"}
             )
-        head_data = head_data.assign(operateType=operate_type)
+            head_data = head_data.assign(
+                headMainId=pd.Series(
+                    [uuid.uuid4().hex for _ in head_data.index],
+                    index=head_data.index
+                )
+            )
+        head_data = head_data.assign(operateType=header_operate)
+
+        # NB: replace twice in case of infer None to nan happened
+        head_df = head_df.replace({None: np.nan})
         head_df = head_df.replace({np.nan: None})
+
         head_data = head_data.assign(
             data=head_df.drop(columns=['_type']).to_dict(orient='records')
         )
@@ -205,7 +228,7 @@ class AsyncJournalModel(ElementBase[JournalModelAPI]):
         self._maybe_raise_errors(errors)
 
         line_data = line_df[line_required_fields]
-        line_data = line_data.assign(operateType=operate_type)
+        line_data = line_data.assign(operateType=line_operate)
 
         # generate lineMainId
         if is_editing:
@@ -222,24 +245,22 @@ class AsyncJournalModel(ElementBase[JournalModelAPI]):
                     index=line_data.index
                 )
             )
+
+        # NB: replace twice in case of infer None to nan happened
+        line_df = line_df.replace({None: np.nan})
         line_df = line_df.replace({np.nan: None})
+
         line_data = line_data.assign(data=line_df.to_dict(orient='records'))
 
         # merge head & line
-        if is_editing:
-            line_data = line_data.merge(
-                head_data[[relation_field, "headMainId", "journalTypeCode"]],
-                on=relation_field,
-            )
-            # relation field is only used for merge
-            head_data = head_data.drop(columns=[relation_field])
-            line_data = line_data.drop(columns=[relation_field])
-        else:
-            line_data = line_data.rename(columns={relation_field: "headMainId"})
-            line_data = line_data.merge(
-                head_data[["headMainId", "journalTypeCode"]],
-                on="headMainId"
-            )
+        line_data = line_data.merge(
+            head_data[[relation_field, "headMainId", "journalTypeCode"]],
+            on=relation_field
+        )
+
+        # relation field is only used for merge
+        head_data = head_data.drop(columns=[relation_field])
+        line_data = line_data.drop(columns=[relation_field])
 
         if callback is not None:
             callback = parse_obj_as(CallbackInfo, callback)
@@ -281,7 +302,7 @@ class AsyncJournalModel(ElementBase[JournalModelAPI]):
         enable_need_one_line: bool = True,
         sync: bool = True
     ) -> CommonResultDTO:
-        """凭证模型数据保存
+        """凭证模型数据新增
         
         Args:
             head_df: 凭证头表的数据（字段名与凭证模型上头表的字段名对应）
@@ -356,8 +377,11 @@ class AsyncJournalModel(ElementBase[JournalModelAPI]):
                 ])
 
                 # 回调脚本
-                callback_info = {"elementName": "testPy01", "elementType": "PY", "path": "/zhy_test",
-                                "callbackParams": {"year": "2023", "period": "03"}}
+                callback_info = {
+                    "elementName": "testPy01", "elementType": "PY",
+                    "path": "/zhy_test",
+                    "callbackParams": {"year": "2023", "period": "03"}
+                }
                 # 创建凭证组件元素对象
                 journal = JournalModel('ZHY_TEST_0613_02')
                 # 调用保存
@@ -372,7 +396,8 @@ class AsyncJournalModel(ElementBase[JournalModelAPI]):
                     enable_valid_range= True,
                     enable_all_errors = True,
                     enable_need_one_line = True,
-                    sync=True)
+                    sync=True
+                )
 
 
         Attention:
@@ -381,14 +406,34 @@ class AsyncJournalModel(ElementBase[JournalModelAPI]):
 
                 .. code-block:: python
 
+                # 凭证组件V1.0.6.3版本以前
+
                 p2 = {
-                   "batchNumber": "b14d943609b",# 批次号
-                   "success": True, # 引擎是否执行成功
-                   "year": "2023", # 自定义参数
-                   "period": "03" # 自定义参数
+                   "batch_id": "b14d943609b",
+                   "success": True,
+                   "year": "2023",  # 自定义参数
+                   "period": "03"   # 自定义参数
                 }
 
-            自定义参数的名称不应当为success或batchNumber，以免和回调脚本传入的默认参数冲突
+                # 凭证组件V1.0.6.3版本及以后
+
+                p2 = {
+                   "mainKey": {
+                        "journal_id": [
+                            "ZDlhYj",
+                            "ZWU00005",
+                            "ZjY4MG",
+                            "NWQ00002",
+                            "ZTl00003",
+                            "NmNiNW",
+                            "YWM5ZG",
+                            "M2E00004"
+                        ]
+                   },
+                   "success": True,
+                   "year": "2023",  # 自定义参数
+                   "period": "03"   # 自定义参数
+                }
 
     """
         batch = self._gen_batch_payload(
@@ -427,6 +472,8 @@ class AsyncJournalModel(ElementBase[JournalModelAPI]):
         enable_valid_range: bool = True,
         enable_all_errors: bool = True,
         enable_need_one_line: bool = True,
+        header_operate: Literal['EDIT', 'DELETE_ADD'] = 'EDIT',
+        line_operate: Literal['EDIT', 'ADD', 'DELETE', 'DELETE_ADD'] = 'EDIT',
     ) -> CommonResultDTO:
         """凭证模型数据更新
 
@@ -447,6 +494,10 @@ class AsyncJournalModel(ElementBase[JournalModelAPI]):
             enable_valid_range: 是否启用有效性范围的校验，默认为True
             enable_all_errors: 是否启用一次性校验所有规则和数据，默认为True
             enable_need_one_line: 是否启用凭证行表至少需要一条数据的校验，默认为True
+            header_operate: 凭证头表的操作类型，默认为EDIT
+            line_operate: 凭证行表的操作类型，默认为EDIT
+                         头表EDIT, 行表可以ADD/EDIT/DELETE
+                         头表DELETE_ADD, 头表update，行表全删全增，行表只能为DELETE_ADD
 
         Returns:
            接口返回信息（CommonResultDTO的success为true 表示成功，如false 则错误在errors集合里）
@@ -456,14 +507,16 @@ class AsyncJournalModel(ElementBase[JournalModelAPI]):
         batch = self._gen_batch_payload(
             head_df=head_df, line_df=line_df,
             callback=callback, relation_field=relation_field,
-            id_col=MAIN_ID, operate_type='EDIT',
+            id_col=MAIN_ID,
             enable_create=enable_create,
             enable_default_value=enable_default_value,
             enable_repeat_check=enable_repeat_check,
             enable_required=enable_required,
             enable_valid_range=enable_valid_range,
             enable_all_errors=enable_all_errors,
-            enable_need_one_line=enable_need_one_line
+            enable_need_one_line=enable_need_one_line,
+            header_operate=header_operate,
+            line_operate=line_operate,
         )
         resp = await self.async_api.journal_model_data.update(batch)
 
@@ -770,6 +823,8 @@ class JournalModel(AsyncJournalModel, metaclass=SyncMeta):
             enable_valid_range: bool = True,
             enable_all_errors: bool = True,
             enable_need_one_line: bool = True,
+            header_operate: Literal['EDIT', 'DELETE_ADD'] = 'EDIT',
+            line_operate: Literal['EDIT', 'ADD', 'DELETE', 'DELETE_ADD'] = 'EDIT',
         ) -> CommonResultDTO:
             ...
 
