@@ -1,8 +1,10 @@
 from enum import Enum
-from typing import List, Dict, Union, Any, Callable, TYPE_CHECKING
+from typing import List, Dict, Union, Any, Callable, TYPE_CHECKING, Optional
 import pandas as pd
 import datetime
 
+from pydantic import Field
+
 from deepfos.api.models.variable import VariableValueDTO, VariableVO, UpdateVariavlesDTO, ValueTypeMapDTO, \
     BaseElementDetail
 from deepfos.api.variable import VariableAPI
@@ -23,13 +25,13 @@ __all__ = [
 
 
 class GlobalVariable(VariableValueDTO):
-    description: dict = {'zh-cn': None, 'en': None}
+    description: Optional[Dict] = {'zh-cn': None, 'en': None}
     scope: int = 1
     status: bool = True
 
 
 class UserVariable(VariableValueDTO):
-    description: dict = {'zh-cn': None, 'en': None}
+    description: Optional[Dict] = {'zh-cn': None, 'en': None}
     scope: int = 2
     status: bool = True
 
@@ -72,17 +74,17 @@ class NumberValueMap(ValueTypeMapDTO):
     #: 变量类型代码
     valueType: int = VariableTypeId.NUMBER
     #: 整数长度
-    maxLen: int = None
+    maxLen: Optional[int] = None
     #: 小数长度
-    digitLen: int = None
+    digitLen: Optional[int] = None
     #: 允许等于最小值
     minEqual: bool = True
     #: 最小值
-    minimun: int = None
+    minimun: Optional[int] = None
     #: 允许等于最大值
     maxEqual: bool = True
     #: 最大值
-    maximun: int = None
+    maximun: Optional[int] = None
     #: 是否显示为百分比
     percentage: bool = False
 
@@ -91,30 +93,30 @@ class SmartListValueMap(ValueTypeMapDTO):
     #: 变量类型代码
     valueType: int = VariableTypeId.SML
     #: 元素详情(由smartlist提取后的元素信息)
-    elementDetail: BaseElementDetail = None
+    elementDetail: Optional[BaseElementDetail] = None
     #: 值列表元素名称
-    valueKey: str = None
+    valueKey: Optional[str] = None
     #: 是否选择多个值列表成员
     selectedMulti: bool = False
     #: 多选值列表的成员值列表
-    valueField: List[str] = []
+    valueField: List[str] = Field(default_factory=list)
     #: 多选成员数上限
-    multipleChoiceLimit: int = None
+    multipleChoiceLimit: Optional[int] = None
 
 
 class DimensionValueMap(ValueTypeMapDTO):
     #: 变量类型代码
     valueType: int = VariableTypeId.DIM
     #: 元素详情(由Dimension提取后的元素信息)
-    elementDetail: BaseElementDetail = None
+    elementDetail: Optional[BaseElementDetail] = None
     #: 维度元素名称
-    dimensionName: str = None
+    dimensionName: Optional[str] = None
     #: 维度表达式
-    valueKey: str = None
+    valueKey: Optional[str] = None
     #: 是否选择多个值列表成员
     selectedMulti: bool = False
     #: 多选成员数上限
-    multipleChoiceLimit: int = None
+    multipleChoiceLimit: Optional[int] = None
 
 
 class DateValueMap(ValueTypeMapDTO):
@@ -128,11 +130,11 @@ class CustomListValueMap(ValueTypeMapDTO):
     #: 变量类型代码
     valueType: int = VariableTypeId.CUSTOM_LIST
     #: 自定义列表表达式(多个值请用“,”分开。示例: 1,2,3)
-    valueKey: str = None
+    valueKey: Optional[str] = None
     #: 是否选择多个值列表成员
     selectedMulti: bool = False
     #: 多选成员数上限
-    multipleChoiceLimit: int = None
+    multipleChoiceLimit: Optional[int] = None
 
 
 # -----------------------------------------------------------------------------
@@ -182,11 +184,11 @@ class AsyncVariable(ElementBase[VariableAPI]):
         path: str = None,
         server_name: str = None,
     ):
-        super().__init__(element_name=element_name, folder_id=folder_id, path=path, server_name=server_name)
         self.__group = Group()
         self.__description = None
         self.__gv_memo = None
         self.__uv_memo = None
+        super().__init__(element_name=element_name, folder_id=folder_id, path=path, server_name=server_name)
 
     @property
     def _group(self) -> Group:
@@ -332,6 +334,10 @@ class AsyncVariable(ElementBase[VariableAPI]):
         if obj_hook is not None:
             return obj_hook(value)
 
+        # 启用维度表达式的维度变量应直接取原值
+        if var.valueType == VariableTypeId.DIM and not var.valueTypeMap.selectedMulti:
+            return value
+
         if not auto_cast:
             cast = None
         elif var.valueType == VariableTypeId.NUMBER:
