@@ -2,8 +2,9 @@ from functools import partial
 from typing import *
 
 import pandas as pd
-from pydantic import parse_obj_as
 
+from deepfos.api.models import compat_parse_obj_as as parse_obj_as
+from deepfos.api.models.platform import FileUplodRes
 from deepfos.api.models.workflow import *
 from deepfos.api.workflow import WorkFlowAPI
 from deepfos.element.base import ElementBase, SyncMeta
@@ -295,22 +296,76 @@ class AsyncWorkFlow(ElementBase[WorkFlowAPI]):
         comment: str = None,
         file_path: str = None,
         outcome: str = None,
+        extra_res_params: Dict[str, Any] = None,
+        attachments: List[Union[Dict[str, str], FileUplodRes]] = None,
     ) -> bool:
         """完成任务实例
 
         Args:
             task_id: 任务实例id
             comment: 备注
-            file_path: 附件路径
+            file_path: 附件路径(工作流版本在V3.0.4.0后弃用，后续版本请使用attachments参数)
             outcome: 结果选项，在任务可选结果不唯一时，必须提供
+            extra_res_params: 完成参数
+            attachments: 附件信息列表，
+                        内容一般来自文件管理的/files/upload或/files/upload/content响应值
+
 
         Returns:
             True: 成功
             False: 失败
 
+        .. admonition:: 示例
+
+            .. code-block:: python
+
+                from deepfos.api.platform import PlatformAPI
+                from deepfos.element.workflow import WorkFlow
+
+                # 上传附件't.txt'
+                upload_resp = PlatformAPI().file.upload(
+                    file_type='DL', file_name='t.txt', file='some text'
+                )
+
+                # 以动作approve，参数{"a": 1, "b": "42"}完成任务实例，
+                # 并提供附件信息为上传的't.txt'文件，备注为"Completed by SDK"
+                test_task = WorkFlow('test_task')
+                test_task.complete_task_by_id(
+                    task_id='fd94f6a7-3467-47f9-8a3c-ff626e68dcf5',
+                    outcome='approve',
+                    extra_res_params={'a': 1, 'b': '42'},
+                    comment='Completed by SDK',
+                    attachments=[upload_resp]
+                )
+
         """
+        action_id = None
+        if outcome is not None:
+            outcomes = await self.async_api.task.outcomes(task_id)
+            for o in outcomes:
+                if o.code == outcome:
+                    action_id = o.id
+                    break
+
+        if extra_res_params is None:
+            extra_res_params = {}
+        if not isinstance(extra_res_params, dict):
+            raise TypeError('extra_res_params参数应为字典类型')
+
+        if attachments is not None:
+            attachments = parse_obj_as(List[FileUplodRes], attachments)
+        else:
+            attachments = []
         return await self.async_api.task.express_complete(
-            TaskCompleteInstance(comment=comment, filePath=file_path, outcome=outcome, taskId=task_id)
+            TaskCompleteInstance(
+                comment=comment, filePath=file_path, outcome=outcome, taskId=task_id,
+                extraResParams=[
+                    TaskCompleteParam(name=k, value=v)
+                    for k, v in extra_res_params.items()
+                ],
+                actionId=action_id,
+                attachments=attachments
+            )
         )
 
     async def list_process(
@@ -536,6 +591,8 @@ class WorkFlow(AsyncWorkFlow, metaclass=SyncMeta):
             comment: str = None,
             file_path: str = None,
             outcome: str = None,
+            extra_res_params: Dict[str, Any] = None,
+            attachments: List[Union[Dict[str, str], FileUplodRes]] = None,
         ) -> bool:
             ...
 
