@@ -2,7 +2,6 @@ from __future__ import annotations
 
 import asyncio
 import itertools
-from asyncio import SelectorEventLoop
 import asyncio.events as events
 from asyncio import Future as AioFuture
 import sys
@@ -18,7 +17,7 @@ from reprlib import aRepr
 from typing import (
     Optional, Tuple, Union, TYPE_CHECKING, List,
     Callable, NamedTuple, Coroutine, Any, Dict,
-    Generic, Iterable, TypeVar, Awaitable, overload,
+    Generic, Iterable, TypeVar, Awaitable, overload, Container, Set,
 )
 # noinspection PyProtectedMember
 from concurrent.futures._base import (
@@ -102,10 +101,18 @@ def _patch_task():
 
 _patch_task()
 
+if sys.platform == 'win32':
+    BaseEventLoop = asyncio.ProactorEventLoop
+    is_proactorloop = True
+else:
+    BaseEventLoop = asyncio.SelectorEventLoop
+    is_proactorloop = False
+
 
 # noinspection PyUnresolvedReferences,PyAttributeOutsideInit,PyProtectedMember
-class ReentrantLoop(SelectorEventLoop):
+class ReentrantLoop(BaseEventLoop):
     _num_runs_pending = 0
+    _is_proactorloop = is_proactorloop
     _nest_patched = True
 
     def run_forever(self):
@@ -140,11 +147,22 @@ class ReentrantLoop(SelectorEventLoop):
             self._thread_id = threading.get_ident()
             events._set_running_loop(self)
             self._num_runs_pending += 1
+            if self._is_proactorloop:
+                if self._self_reading_future is None:
+                    self.call_soon(self._loop_self_reading)
             yield
         finally:
             self._thread_id = old_thread_id
             events._set_running_loop(old_running_loop)
             self._num_runs_pending -= 1
+            if self._is_proactorloop:
+                if (self._num_runs_pending == 0
+                        and self._self_reading_future is not None):
+                    ov = self._self_reading_future._ov
+                    self._self_reading_future.cancel()
+                    if ov is not None:
+                        self._proactor._unregister(ov)
+                    self._self_reading_future = None
 
     @contextmanager
     def manage_asyncgens(self):
@@ -200,6 +218,10 @@ class ReentrantLoop(SelectorEventLoop):
         handle = None  # noqa
 
 
+class ReentrantLoopPolicy(events.BaseDefaultEventLoopPolicy):  # noqa
+    _loop_factory = ReentrantLoop
+
+
 # -----------------------------------------------------------------------------
 # core
 class Node(Generic[NodeT]):
@@ -374,7 +396,12 @@ class EventLoop:
             self.daemon = True
             self.id = None
             self._bind = weakref.ref(bind)
-            self.loop = ReentrantLoop()
+            default_policy = asyncio.get_event_loop_policy()
+            try:
+                asyncio.set_event_loop_policy(ReentrantLoopPolicy())
+                self.loop = asyncio.new_event_loop()
+            finally:
+                asyncio.set_event_loop_policy(default_policy)
 
         def run(self):
             try:
@@ -626,7 +653,7 @@ class _TaskProp:
 # noinspection PyPep8Naming
 class future_property(Generic[ReturnT]):
     def __init__(
-        self, 
+        self,
         func: Optional[Callable[[Any], Coroutine[ReturnT, Any, ReturnT]]] = None,
         on_demand: bool = False
     ):
@@ -692,6 +719,11 @@ class future_property(Generic[ReturnT]):
         if self.attrname in instance.__dict__:
             return instance.__dict__[self.attrname]
 
+        await instance.__future_guard__.wait()
+
+        if self.attrname in instance.__dict__:  # NB: await后任务可能已经结束
+            return instance.__dict__[self.attrname]
+
         task = self._tasks[instance]
         await task.done.wait()
         return task.result
@@ -713,30 +745,57 @@ class future_property(Generic[ReturnT]):
 
 class FuturePropertyMeta(type):
     def __init__(cls, name, bases, namespace):
-        submit_calls = []
+        submit_calls = {}
+
+        call_super = any(isinstance(clz, FuturePropertyMeta) for clz in bases)
 
         for _name, attr in cls.__dict__.items():
             if not isinstance(attr, future_property):
                 continue
-            submit_calls.append(attr.submit)
+            submit_calls[_name] = attr.submit
 
         ori_init = cls.__init__
 
         if ori_init is None:
             def new_init(self, *_args, **_kwargs):
-                for call in submit_calls:
-                    call(self)  # noqa
+                self.__fire_futures__(set())
 
         else:
             def new_init(self, *_args, **_kwargs):
                 ori_init(self, *_args, **_kwargs)
-                for call in submit_calls:
-                    call(self)  # noqa
+                self.__fire_futures__(set())
 
         cls.__init__ = new_init
+        cls.__future_fired__ = False
         if callable(ori_init):
             cls.__init__.__signature__ = inspect.signature(ori_init)
 
+        def __fire_futures__(self, skip: Set[str], is_root_call: bool = True):
+            if is_root_call and not hasattr(self, '__future_guard__'):
+                if evloop.in_same_thread():
+                    self.__future_guard__ = asyncio.Event()
+                else:
+                    async def attch_guard():
+                        self.__future_guard__ = asyncio.Event()
+                    evloop.run(attch_guard())
+
+            if self.__future_fired__:
+                return
+
+            if call_super:
+                # NB: 必须先调用super, 因为子类的future可能依赖父类，如果后调用super，可能导致
+                # 子类future运行时父类还未提交，触发task中的KeyError
+                super(cls, self).__fire_futures__(skip.union(submit_calls), False)  # noqa
+
+            for _n, call in submit_calls.items():
+                if _n not in skip:
+                    call(self)  # noqa
+
+            if is_root_call:
+                self.__future_fired__ = True
+                evloop.loop.call_soon_threadsafe(self.__future_guard__.set)
+
+        cls.__fire_futures__ = __fire_futures__
         super().__init__(name, bases, namespace)
 
 
