@@ -3,9 +3,18 @@ import re
 
 UNSET = object()
 DFLT_DATA_COLUMN = 'data'
+COLUMN_USAGE_FIELD = 'VirtualMeasure_220922'
+DFLT_COMMENT_COLUMN = 'VirtualMeasure_220922'
+USED_FOR_COMMENT = 'comment'
+USED_FOR_DATA = 'data'
 DFLT_NAME_COLUMN = 'name'
 DFLT_PNAME_COLUMN = 'parent_name'
 SHAREDMEMBER = 'sharedmember'
+
+SHAREDMEMBERV12 = 'sharedMember'
+DFLT_PNAME_COLUMN_V12 = 'parentName'
+
+
 VIEW = "View"
 VIEW_DICT = CaseInsensitiveDict(view=VIEW)
 HIERARCHY = CaseInsensitiveDict(
