@@ -6,8 +6,9 @@ from inspect import isfunction
 from pandas import DataFrame
 from typing import *
 import pandas as pd
-from pydantic import Field, parse_obj_as
+from pydantic import Field
 
+from deepfos.api.models import compat_parse_obj_as as parse_obj_as
 from deepfos.lib.asynchronous import evloop
 from deepfos.api.models import BaseModel
 from deepfos.api.dimension import DimensionAPI
@@ -94,10 +95,10 @@ class LogicInfo(NamedTuple):
 
 
 class BaseField(BaseModel):
-    code: int = None
-    columnName: str = None
-    description: str = None
-    value_key: str = None
+    code: Optional[int] = None
+    columnName: Optional[str] = None
+    description: Optional[str] = None
+    value_key: Optional[str] = None
 
     def __init__(self, name: str = None, description: str = None, **data):
         super().__init__(columnName=name, description=description, **data)
@@ -125,14 +126,14 @@ class BaseField(BaseModel):
 
 
 class Text(BaseField):
-    code = 1
+    code: int = 1
 
 
 class Date(BaseField):
-    code = 11
+    code: int = 11
     # 前端时间筛选器颗粒度字段；4：最宽泛的颗粒度
-    value_key = '4'
-    precision = TimeType.day
+    value_key: str = '4'
+    precision: TimeType = TimeType.day
 
     @property
     def logic_info(self) -> LogicInfo:
@@ -140,11 +141,11 @@ class Date(BaseField):
 
 
 class Number(BaseField):
-    code = 15
+    code: int = 15
 
 
 class Person(BaseField):
-    code = 12
+    code: int = 12
 
 
 class BoundField(BaseField):
@@ -195,7 +196,7 @@ class BoundField(BaseField):
 
 
 class SmartList(BoundField):
-    code = 3
+    code: int = 3
     bound_class: Type[ElementBase] = SmlElement
 
     async def async_to_dict(self) -> Dict:
@@ -209,7 +210,7 @@ class SmartList(BoundField):
 
 
 class Dimension(BoundField):
-    code = 8
+    code: int = 8
     bound_class: Type[ElementBase] = DimElement
 
 
@@ -491,7 +492,7 @@ class Filter:
         # 分组字段：处理存在为空的列重新赋值为 ''，防止记录丢失
         for item in group_by_col:
             if item in df:
-                df.loc[:, item] = df[item].fillna('')
+                df[item] = df[item].fillna('')
 
         if group_by_col and agg_func_col:
             df = df.groupby(group_by_col, as_index=False, sort=False).agg(agg_func_col)
@@ -678,20 +679,20 @@ class ChartEngine:
         for column in self.params.queryInfo.timeColumns:
             column_name = column.columnName
             if column_name in df.columns:
-                df.loc[:, column_name] = df[column_name].astype('datetime64[ns]')
+                df[column_name] = pd.to_datetime(df[column_name])
 
                 if column.formatType == FieldType.year:
-                    df.loc[:, column_name] = df[column_name].dt.strftime('%Y')
+                    df[column_name] = df[column_name].dt.strftime('%Y')
                 elif column.formatType == FieldType.month:
-                    df.loc[:, column_name] = df[column_name].dt.strftime('%Y-%m')
+                    df[column_name] = df[column_name].dt.strftime('%Y-%m')
                 elif column.formatType == FieldType.day:
-                    df.loc[:, column_name] = df[column_name].dt.strftime('%Y-%m-%d')
+                    df[column_name] = df[column_name].dt.strftime('%Y-%m-%d')
                 elif column.formatType == FieldType.quarter:
                     year_col = df[column_name].dt.strftime('%Y-')
                     quarter_col = df[column_name].dt.quarter.astype('str')
-                    df.loc[:, column_name] = year_col + 'Q' + quarter_col
+                    df[column_name] = year_col + 'Q' + quarter_col
 
-                df.loc[:, column_name] = df[column_name].fillna('')
+                df[column_name] = df[column_name].fillna('')
         return df
 
     def _main(self, df: DataFrame):
