@@ -2,9 +2,9 @@ import collections
 import re
 from enum import Enum, IntEnum
 from typing import *
-from pydantic import Field, parse_obj_as
+from pydantic import Field
 
-from deepfos.api.models import BaseModel
+from deepfos.api.models import BaseModel, compat_parse_obj_as as parse_obj_as
 from deepfos.core.logictable.nodemixin import NodeMixin, TreeRenderer
 from deepfos.lib.utils import CIEnum, CIEnumMeta
 
