@@ -177,7 +177,9 @@ class TaskManagerSocket:
     #: 当前还未结束的任务数量
     _pending_tasks: int = 0
 
+    # NB: set port to avoid init args err on win system
     _client = WorkerClient(
+        port=None,
         sockname=OPTION.general.socket_name,
         loop=evloop.loop,
         id=OPTION.general.task_info.get('task_id'),
