@@ -1,5 +1,5 @@
 """系统相关工具类/函数"""
-
+import asyncio
 import base64
 import gzip
 import itertools
@@ -8,7 +8,7 @@ import time
 from datetime import datetime
 from enum import Enum
 from io import TextIOWrapper
-from typing import Dict, Any, Union, List, IO, TypeVar
+from typing import Dict, Any, Union, List, IO, TypeVar, Optional
 import pandas as pd
 import math
 import hashlib
@@ -23,6 +23,7 @@ from deepfos.api.models.deepfos_task import (
     PeriodicTaskCreateInfo, ScheduledTaskCreateInfo
 )
 from deepfos.element.base import ElementBase
+from .asynchronous import evloop
 from .utils import split_dataframe
 from deepfos.options import OPTION
 from deepfos.api.models import BaseModel
@@ -33,6 +34,7 @@ from deepfos.api.system import SystemAPI
 from deepfos.api import account as acc_api
 from deepfos.api.models import account as account_model
 from deepfos.cache import SpaceSeperatedLRUCache
+from deepfos.api.models.account import UserGroupModifyDTO
 
 _VALUE_KEY = 'value'
 _DTNAME_KEY = 'dataTableName'
@@ -102,6 +104,8 @@ def complete_cartesian_product(
     fix: Dict[str, Union[str, list]],
     df: pd.DataFrame = None,
     paths: Union[str, Dict[str, str]] = None,
+    folder_ids: Union[str, Dict[str, str]] = None,
+    col_dim_map: Dict[str, str] = None,
 ) -> pd.DataFrame:
     """
     构造完整的维度成员笛卡尔积
@@ -111,6 +115,8 @@ def complete_cartesian_product(
         df: 如果需要为现有DataFrame补全笛卡尔积，传入一个df。如果不传，则是生成fix中维度成员的笛卡尔积。
         paths: fix中维度的path，如果所有维度的目录相同，传同一个path，否则传字典，key为维度名，value为path。
           如果不传，则自动寻找维度对应的path。
+        folder_ids: 类似paths, 但值是folder_id
+        col_dim_map: data中的列名与实际维度名的映射关系，默认data中的列名与维度名相同
 
     Returns:
         维度成员笛卡尔积的DataFrame
@@ -145,33 +151,49 @@ def complete_cartesian_product(
     from deepfos.element.dimension import AsyncDimension
     from .asynchronous import evloop
 
-    if isinstance(paths, dict):
-        path_getter = paths.__getitem__
+    if paths is not None:
+        loc_key = "path"
+        if isinstance(paths, dict):
+            loc_getter = paths.get
+        else:
+            loc_getter = lambda _: paths
+    elif folder_ids is not None:
+        loc_key = "folder_id"
+
+        if isinstance(folder_ids, dict):
+            loc_getter = folder_ids.get
+        else:
+            loc_getter = lambda _: folder_ids
     else:
-        path_getter = lambda _: paths
+        loc_key = "path"
+        loc_getter = lambda _: None
+    if col_dim_map is None:
+        col_dim_map = {}
 
     # 遍历fix，如果fix的值为str，则认为是维度表达式，将表达式转换为成员list
     mbrs = {}
     futures = []
 
-    for dim, exp in fix.items():
+    for col, exp in fix.items():
         if isinstance(exp, str):
             if "(" not in exp:
-                mbrs[dim] = exp.split(';')
+                mbrs[col] = exp.split(';')
             else:
-                path = path_getter(dim)
-                future = evloop.apply(AsyncDimension(element_name=dim, path=path).query(
+                dim = col_dim_map.get(col, col)
+                loc = loc_getter(dim)
+                future = evloop.apply(AsyncDimension(element_name=dim, **{loc_key: loc}).query(
                     expression=exp, fields=['name'], as_model=False
                 ))
 
-                futures.append((dim, future))
+                futures.append((col, future))
         else:
             if not isinstance(exp, list):
                 raise TypeError('fix参数的value只能为维度表达式(str)或维度成员(list)')
-            mbrs[dim] = exp
+            mbrs[col] = exp
 
-    for dim, future in futures:
-        mbrs[dim] = [item['name'] for item in future.result()]
+    for col, future in futures:
+        dim_mbrs = future.result()
+        mbrs[col] = [item['name'] for item in dim_mbrs]
 
     if df is None:
         df = pd.DataFrame(columns=list(fix.keys()))
@@ -225,10 +247,17 @@ def export_file_for_download(file_name: str, file: Union[str, bytes, TextIOWrapp
 
             # . 提供buffer
             import io
+            import pandas as pd
 
             buffer = io.BytesIO()
-            buffer.write(b'8888')
-            export_file_for_download('t4.txt', buffer.getbuffer())
+            # 将dataframe内容写入buffer
+            with pd.ExcelWriter(buffer, engine="openpyxl") as writer:
+                  pd.DataFrame({'a': [1, 2, 3], 'b': [4, 5, 6]}).to_excel(
+                    writer, index=False, encoding='utf-8'
+                  )
+
+            # 上传至下载中心
+            export_file_for_download('out.xlsx', buffer.getbuffer())
 
 
     """
@@ -450,14 +479,14 @@ class TaskUtil:
         py_info: 任务实例对应的python元素信息，如未提供，仍可在 `run_job_contents` 中提供
 
     """
-    api = TaskAPI(sync=True)
-    _chunksize = 200
 
     def __init__(self, task_code, py_info: PyInfo = None):
         self.task_code = task_code
+        self.api = TaskAPI(sync=True)
         if self.meta is None:
             raise ValueError(f"No config for task_code: [{self.task_code}].")
         self.py_info = py_info
+        self._chunksize = 200
 
     @cached_property
     def meta(self):
@@ -550,18 +579,17 @@ class TaskUtil:
         if not_found_col:
             raise ValueError(f'Required columns:{sorted(not_found_col)} since they are compositeKeys.')
 
-    @classmethod
-    def _create_task_instance(cls, call_api, job_contents, payload):
-        if job_contents.shape[0] <= cls._chunksize:
+    def _create_task_instance(self, call_api, job_contents, payload):
+        if job_contents.shape[0] <= self._chunksize:
             payload.jobContent = job_contents.to_dict(orient='records')
             payload.lastBatch = True
             call_api(payload)
         else:
-            payload.jobContent = job_contents.iloc[0:cls._chunksize:].to_dict(orient='records')
+            payload.jobContent = job_contents.iloc[0:self._chunksize:].to_dict(orient='records')
             payload.batchId = call_api(payload).batchId
             payloads = []
 
-            for batch_contents in split_dataframe(job_contents.iloc[cls._chunksize::], cls._chunksize):
+            for batch_contents in split_dataframe(job_contents.iloc[self._chunksize::], self._chunksize):
                 payload.jobContent = batch_contents.to_dict(orient='records')
                 payloads.append(payload)
 
@@ -658,3 +686,37 @@ def resolve_account_api_extra_header():
         'timestamp': timestamp,
         'sign': calc_account_api_signature(timestamp, platform_secret, platform_code)
     }
+
+
+def batch_modify_user_group(payloads: List[UserGroupModifyDTO], max_worker: int = None):
+    """批量调用用户中心用户组详情修改接口
+
+    Args:
+        payloads: 符合UserGroupModifyDTO定义的列表，将直接用作接口请求体
+        max_worker: 最大并发数
+
+    Returns: 与请求体顺序一致的返回结果列表
+
+    """
+    if max_worker is not None:
+        if max_worker <= 0:
+            raise ValueError('max_worker must be > 0 ')
+    else:
+        max_worker = len(payloads)
+
+    result: List[Optional[bool]] = [None] * len(payloads)
+    api = acc_api.AccountAPI(sync=False).user_group.space_modify_group
+
+    async def call_api(idx: int, p: UserGroupModifyDTO, sem: asyncio.Semaphore):
+        async with sem:
+            result[idx] = await api(p)
+
+    async def inner():
+        semaphore = asyncio.Semaphore(max_worker)
+        await asyncio.gather(*(
+            call_api(idx, payload, semaphore)
+            for idx, payload in enumerate(payloads)
+        ))
+
+    evloop.run(inner())
+    return result
