@@ -7,13 +7,14 @@ import sys
 import time
 import weakref
 from collections import UserList, UserDict, defaultdict
+from contextlib import contextmanager, nullcontext
 from enum import EnumMeta, Enum
 import random
 from typing import (
     Tuple, Optional, Dict,
     List, Union, Callable, Any,
     TypeVar, MutableMapping, Container,
-    Iterator, Iterable, DefaultDict
+    Iterator, Iterable, DefaultDict,
 )
 from itertools import groupby, count
 
@@ -867,6 +868,38 @@ def dict_to_sql(
     return sql
 
 
+class ChunkAlert:
+    def __call__(self, start: int, end: int, exc: Exception = None) -> None: ...
+
+
+@contextmanager
+def chunk_alert(
+    start: int, end: int,
+    before: ChunkAlert = None,
+    after: ChunkAlert = None,
+):
+    try:
+        if before is not None:
+            try:
+                before(start, end)
+            except Exception:
+                logger.warning('Error occurs while calling before_chunk.')
+        yield
+    except Exception as e:
+        if after is not None:
+            try:
+                after(start, end, e)
+            except Exception:
+                logger.warning('Error occurs while calling after_chunk.')
+        raise
+    else:
+        if after is not None:
+            try:
+                after(start, end)
+            except Exception:
+                logger.warning('Error occurs while calling after_chunk.')
+
+
 def split_dataframe(data: pd.DataFrame, chunksize: int = None):
     nrows = len(data)
     if chunksize is None or chunksize > nrows:
@@ -878,6 +911,33 @@ def split_dataframe(data: pd.DataFrame, chunksize: int = None):
             yield data.iloc[i: i + chunksize]
 
 
+def split_dataframe_alert(
+    data: pd.DataFrame,
+    chunksize: int = None,
+    before_chunk: ChunkAlert = None,
+    after_chunk: ChunkAlert = None,
+):
+    no_alert = before_chunk is None and after_chunk is None
+
+    nrows = len(data)
+    if chunksize is None or chunksize > nrows:
+        if no_alert:
+            yield data, nullcontext()
+        else:
+            yield data, chunk_alert(0, nrows, before_chunk, after_chunk)
+    elif chunksize <= 0:
+        raise ValueError("chunksize must be greater than 0.")
+    else:
+        for i in range(0, nrows, chunksize):
+            if no_alert:
+                yield data.iloc[i: i + chunksize], nullcontext()
+            else:
+                yield (
+                    data.iloc[i: i + chunksize],
+                    chunk_alert(i, min(i + chunksize, nrows), before_chunk, after_chunk)
+                )
+
+
 def find_str(
     target: str,
     candidates: Iterable[str],
@@ -903,19 +963,20 @@ def find_str(
             return candidate
 
 
-def to_version_tuple(ver: Union[float, str]):
+def to_version_tuple(ver: Union[float, str], max_split: int = 1):
     """返回版本元组
 
     Args:
         ver: 表示版本的字符串或数字，例如'1.0'，'1_0'，1.0
+        max_split: 以'.'和'_'分割的最大次数，如为-1，则无限制
 
     Returns:
         形如(1, 0)的版本元组
     """
     if isinstance(ver, float):
         ver = str(ver)
-    version_parts = ver.replace('.', '_').split('_', 1)
-    version_list = [int(part) for part in version_parts]
+    version_parts = ver.replace('.', '_').split('_', max_split)
+    version_list = [int(part) if part.isdigit() else 0 for part in version_parts]
     return tuple(version_list)
 
 
@@ -992,3 +1053,11 @@ class AliasGenerator(SimpleCounter):
     def get(self, hint: str = '') -> str:
         idx = self.nextval(hint)
         return f'{hint}{idx}'
+
+
+def trim_text(content: str, limit: Union[int, None], appendor: str = '...') -> str:
+    limit = limit or 0
+    if 0 < limit < len(content):
+        return f"{content[:limit]} {appendor}"
+    else:
+        return content
