@@ -1,5 +1,6 @@
 import configparser
 import sys
+import threading
 import warnings
 from enum import Enum
 from functools import partial
@@ -276,6 +277,10 @@ class _Option(Generic[T_Opt]):
             self.unset_action = _Action.PASS
         self.unset = False
 
+    def _quick_set(self, instance, value):
+        setattr(instance, self._var_name, value)
+        self.unset = False
+
     def __str__(self):
         return self._display_name
 
@@ -366,7 +371,7 @@ class _Category:
         setattr(self, '_name', getattr(other, '_name'))
         for attr, option in self._options.items():
             try:
-                option.__set__(self, getattr(other, attr))
+                option._quick_set(self, getattr(other, attr))  # noqa
             except BaseOptionError:
                 continue
 
@@ -425,6 +430,7 @@ class _General(_Category):
     parallel_mode = _Option(False, val_type=bool)
     socket_name = _Option(None, val_type=str)
     preserve_concurrency = _Option(1, val_type=int)
+    response_display_length_on_error = _Option(20000, val_type=int)
 
     def __get__(self, instance, owner) -> '_General':
         """defined to help ide"""
@@ -482,6 +488,12 @@ class _ServiceDiscovery(_Category):
     enabled = _Option(False, val_type=bool, on_set=_ensure_discovery_server_is_set)
     #: 服务注册发现使用的实现
     implementation = _Option('eureka', val_type=str, val_choices=('eureka', 'nacos'))
+    #: 服务注册发现使用的缓存策略
+    cache_strategy = _Option(
+        'ranked',
+        val_type=str,
+        val_choices=('ranked', 'roundrobin', 'random')
+    )
 
     def __get__(self, instance, owner) -> '_ServiceDiscovery':
         """defined to help ide"""
@@ -636,19 +648,31 @@ class _OptionCTX:
         boost: _Boost
         discovery: _ServiceDiscovery
         nacos: _Nacos
+        edgedb: _Edgedb
 
         def show_options(self, category=None):
             ...
 
     def __init__(self):
-        self._token = _option_ctx.set(_GlobalOptions())
+        self._create_at_main = threading.current_thread() is threading.main_thread()
+        self._primary_opt = _GlobalOptions()
+        self._token = _option_ctx.set(self._primary_opt)
 
     def create_local(self):
         self._token = _option_ctx.set(_GlobalOptions())
 
     @property
     def _option(self) -> _GlobalOptions:
-        return _option_ctx.get()
+        try:
+            return _option_ctx.get()
+        except LookupError:
+            if (
+                not self._create_at_main
+                and threading.current_thread() is threading.main_thread()
+            ):
+                _option_ctx.set(self._primary_opt.__copy__())
+                return _option_ctx.get()
+            raise
 
     def __getattr__(self, item):
         return getattr(self._option, item)
